## ----------------------------------------------------------------------------
# Rishu::Proxy
#   里珠を外部プロセスにする.
# -----------------------------------------------------------------------------
# 通常の on_load/on_reqeust/on_unload の代わりに,
# このパッケージの on_load/on_request/on_unload を使うようにしてください.
# -----------------------------------------------------------------------------
# Mastering programed by YAMASHINA Hio
#
# Copyright 2002 YMIRLINK,Inc.
# -----------------------------------------------------------------------------
# $Id: Proxy.pm,v 1.4 2002/11/04 14:15:05 hio Exp $
# -----------------------------------------------------------------------------
package Rishu::Proxy;

# -----------------------------------------------------------------------------
# &handler
#  なゆき 委譲 export
sub handler
{
  (\&on_load,\&on_request,\&on_unload);
}

# -----------------------------------------------------------------------------
# on_load/on_reqeust/on_unload の受信部分.
# -----------------------------------------------------------------------------
use vars qw($DATA);

# -----------------------------------------------------------------------------
# &on_load
sub on_load
{
  $DATA = Rishu::Proxy->new();
  $DATA->connect() or return undef;
  my $res = $DATA->transport('LOAD',"Directory: $_[0]\r\n\r\n");
  $res =~ m,^([^\s/]+)/(\d\.\d+)\s+(\d+)\s+(.*), or return undef;
  my $code = $3;
  int($code/100)==2;
}

# -----------------------------------------------------------------------------
# &on_request
sub on_request
{
  $DATA or return '';
  my $res = $DATA->transport('REQUEST','Command: '.$_[0]);
  $res =~ s/([^\r\n]*)\r?\nOriginal-Response:\s+//i;
  $res;
}

# -----------------------------------------------------------------------------
# &on_unload
sub on_unload
{
  $DATA or return 1;
  my $res = $DATA->transport('UNLOAD',"\r\n");
  undef $DATA;
  $res =~ m,^([^s/]+)/(\d\.\d+)\s+(\d+)\s+(.*), or return undef;
  my $code = $3;
  int($code/100)==2;
}

# -----------------------------------------------------------------------------
# Proxy 実装
# -----------------------------------------------------------------------------

# -----------------------------------------------------------------------------
# Rishu::Proxy->new 
sub new
{
  my $pkg = shift;
  my $this = bless {},$pkg;
  
  $this->{exec} = 'rishu_remote.pl';
  
  $this;
}

# -----------------------------------------------------------------------------
# DESTROY
sub DESTROY
{
  my $this = shift;
  kill 9,$this->{pid_child};
}

# -----------------------------------------------------------------------------
# $proxy->log('name','message');
sub log
{
  #return;
  my $this = shift;
  my $name = shift;
  my $message = shift;
  print "(( $name ))\n";
  $message =~ s/^/(( /g;
  print $message;
}

# -----------------------------------------------------------------------------
# $proxy->connect()
sub connect
{
  my $this = shift;
  
  my ($par_rd,$par_wd,$chl_rd,$chl_wd) = (gensym(),gensym(),gensym(),gensym());
  pipe($par_rd,$chl_wd);
  pipe($chl_rd,$par_wd);
  $this->{pid_child} = fork();
  defined($this->{pid_child}) or return undef;
  if( $this->{pid_child} )
  { # parent
    ($this->{rd},$this->{wd}) = ($par_rd,$par_wd);
    ($this->{cr},$this->{cw}) = ($chl_rd,$chl_wd);
    select((select($this->{wd}),$|=1)[0]);
    1;
  }else
  { # child
    open(STDIN, "<&=".fileno($chl_rd));
    open(STDOUT,">&".fileno($chl_wd));
    open(STDERR,">&".fileno($chl_wd));
    
    my $perl = 'modules/5.8.0/bin/MSWin32-x86-multi-thread/perl.exe';
    $^O eq 'MSWin32' and $perl =~ s,/,\\,g;
    -x $perl or $perl = 'perl';
    {exec($perl,$this->{exec})};
    print "exec failed: $!\r\n\r\n";
    exit;
  }
}

# -----------------------------------------------------------------------------
# $proxy->transport
sub transport
{
  my $this = shift;
  my $type = shift;
  my $body = shift;
  
  $this->log('msg',"<< send: $type >>\n$body----\n");
  
  my $wd = $this->{wd};
  print $wd "PROXY $type RISHU/1.1\r\n$body";
  
  my $res = '';
  READ:
  {
    my $rd = $this->{rd};
    while(<$rd>)
    {
      s/\r?\n//;
      $res .= "$_\r\n";
      $this->log('msg',"got: $_\n");
      $_ eq '' and last READ;
    }
    # 切断された.
    $this->clear();
  }
  
  $res;
}

# -----------------------------------------------------------------------------
# $proxy->clear()
sub clear
{
  my $this = shift;
  
  # なんとなくおしまいって送ってみる
  my $sym = $this->{wd};
  print $sym "PROXY EXIT PROXY/1.0\r\n";
  
  $this->{wd} ne \*STDOUT and close $this->{wd};
  $this->{rd} ne \*STDIN  and close $this->{rd};
  close $this->{pid_child};
  delete $this->{wd};
  delete $this->{rd};
  delete $this->{pid_child};
  kill 9,$this->{pid_child};
  
  $this;
}


# -----------------------------------------------------------------------------
# &gensym
#
# (from Symbol.pm)
#
# Note that we never _copy_ the glob; we just make a ref to it.
# If we did copy it, then SVf_FAKE would be set on the copy, and
# glob-specific behaviors (e.g. C<*$ref = \&func>) wouldn't work.
#

my $genpkg = "Rishu::Proxy::Gensym::";
my $genseq = 0;

sub gensym
{
  my $name = 'GEN'.$genseq++;
  my $ref = \*{$genpkg.$name};
  delete $$genpkg{$name};
  $ref;
}

# -----------------------------------------------------------------------------
# end of package
# -----------------------------------------------------------------------------
1;
